function NNF = AggregateNNF(NNF, fi, si)

global maxRansacFeatures;

[h, w, ~] = size(NNF);
[X, Y] = meshgrid(0:(w-1), 0:(h-1));


alphaPlus = ReadFromBinaryFile(alphaPlusFile(fi, si));
alphaMinus = ReadFromBinaryFile(alphaMinusFile(fi, si));

skipH = sqrt((h*w)/maxRansacFeatures);
skipW = sqrt((h*w)/maxRansacFeatures);

mask = false(h, w);
mask(1:skipH:end, 1:skipW:end) = true;
indsT = sum(alphaPlus>0, 3) == 3 | sum(alphaMinus>0, 3) == 3;

inds = find(indsT & mask);
indsT = find(indsT);

if (length(inds) > 5)

    Xb = NNF(inds);
    Yb = NNF(inds+h*w);
    Xa = X(inds);
    Ya = Y(inds);
    
    sigmaTranslation = 2.5 * sqrt(h*w)/500;
    sigmaSimilarity = 0.5 * sqrt(h*w)/500;
    
    [~, indModelsT] = FitModel(Xa, Ya, Xb, Yb, sigmaTranslation, 'Translation');
    [models, indModels] = FitModel(Xa(indModelsT), Ya(indModelsT), Xb(indModelsT), Yb(indModelsT), sigmaSimilarity, 'similarity');

    if (~isempty(models))
        Xb = NNF(indsT);
        Yb = NNF(indsT+h*w);
        Xa = X(indsT);
        Ya = Y(indsT);

        [~, T_noise_squared] = error_affine([], [], sigmaSimilarity, 0.99);
        XT = double([Xa'; Ya'; Xb'; Yb']);
        [~, indModelsTmp] = get_consensus_set(XT, models, T_noise_squared, @error_affine);
        indModels = find(indModelsTmp);
        if (length(indModels) > 5)
            tr = cp2tform(double([Xa(indModels), Ya(indModels)]), double([Xb(indModels), Yb(indModels)]), 'nonreflective similarity');
            models = tr.tdata.T';
        end
    end


    if(~isempty(models))
        curModel = reshape(models, [3, 3]);
        curInd = indModels;
        newCoord = curModel * [Xa(curInd)';Ya(curInd)';ones(1, length(curInd))];
        Xb = newCoord(1, :)./newCoord(3, :);
        Yb = newCoord(2, :)./newCoord(3, :);
        NNF(indsT(curInd)) = Xb;
        NNF(indsT(curInd)+h*w) = Yb;
    end

end
