function opticalFlow = ComputeOpticalFlow(expAdj)

global havePar;

%%% We use Ce Liu's optical flow algorithm to compute the local flows. All
%%% the parameters are the default parameters.
alpha = 0.012;
ratio = 0.75;
minWidth = 20;
nOuterFPIterations = 7;
nInnerFPIterations = 1;
nSORIterations = 30;
para = [alpha, ratio, minWidth, nOuterFPIterations, nInnerFPIterations, nSORIterations];

src = {expAdj{2}, expAdj{2}, expAdj{3}, expAdj{1}};
trg = {expAdj{1}, expAdj{3}, expAdj{1}, expAdj{3}};

vx = cell(1, 4); vy = cell(1, 4);

if (havePar)
    parfor i = 1 : 4
        [vx{i}, vy{i}, ~] = Coarse2FineTwoFrames(src{i}, trg{i}, para);
    end
else
    for i = 1 : 4
        mLength = fprintf(' %d%% Done', (i-1)/4 * 100);
        [vx{i}, vy{i}, ~] = Coarse2FineTwoFrames(src{i}, trg{i}, para);
        fprintf(repmat('\b', 1, mLength));
    end
end

opticalFlow.left = cat(3, vx{1}, vy{1});
opticalFlow.right = cat(3, vx{2}, vy{2});
opticalFlow.leftRight = cat(3, vx{3}, vy{3});
opticalFlow.rightLeft = cat(3, vx{4}, vy{4});


