function [model, inds] = FitModel(xa, ya, xb, yb, sigma, modelType)

%%% This is a wrapper to use the Zuliani's RANSAC code

X = double([xa'; ya'; xb'; yb']);

%%% RANSAC parameters internal  - see details in RANSAC_MZ
ransac_options.epsilon = 1e-6;
ransac_options.P_inlier = 0.99;
ransac_options.mode = 'MSAC';
ransac_options.Ps = [];
ransac_options.notify_iter = [];
ransac_options.fix_seed = true;
ransac_options.stabilize = false;
ransac_options.verbose = false;
ransac_options.notify_iters = true;
ransac_options.max_iters = 1000;
ransac_options.max_no_updates = ransac_options.max_iters;
ransac_options.theta_avg_inliers = 1;
ransac_options.sigma = sigma;
ransac_options.max_iters = 40000;
ransac_options.max_no_updates = ransac_options.max_iters;
ransac_options.est_fun = @estimate_affine;
ransac_options.man_fun = @error_affine;
if isfield(ransac_options,'est_fun_params')
    ransac_options = rmfield(ransac_options,'est_fun_params');
end
ransac_options.est_fun_params.estimation_quality_1 = 100;
ransac_options.est_fun_params.transform_type = modelType;
ransac_options.final_sample_set = 5;


%%% ---------- RANSAC estimation -------------------------
ind_valid = 1:size(X,2);
X = double([xa'; ya'; xb'; yb']);

ransac_results = RANSAC(X(:,ind_valid), ransac_options);

if isempty(ransac_results.Theta), 
    error('Could not fine any model based on the matches');
end

inds = ind_valid(ransac_results.CS);
model = ransac_results.Theta;

%%% -------------------------------------------------------


