function GenerateHDRVideo(vidObj, expoTimes, sceneName)

%%% Throughout the code the following variables are used to indicate frame,
%%% reference, and scale indexes.
%%% fi --> frame index
%%% ri --> reference index
%%% si --> scale index


%%% Declaring global variables -----------------
global minImgSize;
global numScales;
global DSMethod;
global refInd;
global havePar;
global outFinalDir;
%%% --------------------------------------------

t = tic;

%%% Initializing the values --------------------
minSizeDim = min(vidObj.h, vidObj.w);

if (numScales ~= 1)
    resizeFac = (minSizeDim/minImgSize).^(1/(numScales-1));
else
    resizeFac = 1;
end
%%% --------------------------------------------

%%% Generating multi-scale Images --------------
mLength = 0;
fprintf('\nGenerating input pyramids ');

for fi = 1 : vidObj.numFrames
    ri = refInd(fi, vidObj.numExposures);
    inputImg = LoadFrame(vidObj, fi);
    
    for si = 1 : numScales
        curImg = Clamp(imresize(inputImg,1/resizeFac^(si-1),DSMethod), 0, 1);
        WriteToBinaryFile(targetFile(fi, si, ri), curImg);
    end
    
    fprintf(repmat('\b', 1, mLength));
    mLength = fprintf('%d%%', int32(fi/vidObj.numFrames * 100));
end

fprintf(repmat('\b', 1, mLength));
fprintf('Done');
%%% --------------------------------------------

%%% Generating the initial flow and search map--------------------
fprintf('\nGenerating the initial flows and search maps ...');

startTime = clock;
m1Length = 0;

v = ver;
havePar = any(strcmp('Parallel Computing Toolbox', {v.Name}));

if (havePar && matlabpool('size') == 0)
    evalc('matlabpool open');
end


% There is no frame before the first frame and after the last frame. So
% we compute the initial motion from frame 2 to end-1.
for fi = 2 : vidObj.numFrames - 1

    if (fi ~= 2)
        fprintf(repmat('\b', 1, m1Length + mLength));
        mLength = fprintf('\nWorking on frame %d/%d -- Time remaining %s', fi-1, vidObj.numFrames - 2, Sec2Str( ( (vidObj.numFrames - 2)/(fi-1) - 1) * etime(clock, startTime) ) );
    else
        mLength = fprintf('\nWorking on frame %d/%d', fi-1, vidObj.numFrames - 2);
    end
    
    imgs.left = LoadFrame(vidObj, fi-1);
    imgs.trg = LoadFrame(vidObj, fi);
    imgs.right = LoadFrame(vidObj, fi+1);
    
    curExpoTimes(1) = expoTimes(refInd(fi-1, vidObj.numExposures));
    curExpoTimes(2) = expoTimes(refInd(fi, vidObj.numExposures));
    curExpoTimes(3) = expoTimes(refInd(fi+1, vidObj.numExposures));
    
    m1Length = fprintf('\nInitial motion estimation');
    [nnf, flow] = InitialMotionEstimation(imgs, curExpoTimes);
    m1Length = m1Length + fprintf(' -> Done');
    
    m1Length = m1Length + fprintf('\nComputing search map');
    searchMap = ComputeSearchMapBoth(nnf, flow, imgs, curExpoTimes);
    m1Length = m1Length + fprintf(' -> Done');
    
    GenerateMultiScaleData(nnf, searchMap, resizeFac, fi);

end

fprintf(repmat('\b', 1, mLength + m1Length + 3));
fprintf('Done');

if (havePar && matlabpool('size') ~= 0)
   evalc('matlabpool close');
end

mLength = 0;
fprintf('\nPreparing initial required pyramids ');

for fi = 1 : vidObj.numFrames       
    Initialization(vidObj, resizeFac, expoTimes, fi);
    fprintf(repmat('\b', 1, mLength));
    mLength = fprintf('%d%%', int32(fi/vidObj.numFrames * 100));
end

fprintf(repmat('\b', 1, mLength));
fprintf('Done\n\n');
%%% ------------------------------------------------

fid = fopen([outFinalDir, '/timing.txt'], 'wt');
fprintf(fid, 'Timing for the first two steps: %f minute(s)/frame\n', toc(t)/(60*vidObj.numFrames));

t = tic;

%%% Reconstruct HDR video (The third step in the paper) ----
ReconstructHDRVideo(vidObj, expoTimes, resizeFac, sceneName);
%%% ------------------------------------------------

fprintf(fid, 'Timing for the third step: %f minute(s)/frame\n', toc(t)/(60*vidObj.numFrames));
fclose(fid);

