function InitParams(profile)

%%% DO NOT CHANGE ANY PARAMETER, UNLESS YOU KNOW EXACTLY WHAT EACH
%%% PARAMETER DOES.

%%% Declaring global variables -----------------------
global patch_w;                 % patch size
global minImgSize;              % lowest scale resolution size for min(w, h)
global maxIter;                 % number of iterations at the coarsest scale
global minIter;                 % minimum number of iterations at the finest scale
global iterStep;                % the step size for reducing the number of iterations at every scale
global searchVoteInner;         % number of inner iterations
global decreaseIterFromScale;   % the scale in which the number of inner iterations is reduced
global numScales;               % number of scales (distributed logarithmically)
global DSMethod;                % downsampling method
global saveIntermediateResults; % set true to save intermediate results
global vMax;                    % valid color values are between vMin and vMax
global vMin;                    % 
global maxImVal;                % maximum brightness in the input image. For 16 bit images this value is 2^16-1.
global maxRansacFeatures;       % maximum number of correspondence to run RANSAC on. If the number of correspondence is more than this value the code selects a subset of them.
global maxImgSize;              % maximum size of image to find corresponcences. If the images are bigger than this size they will be downsampled.
global cores;                   % number of cores for multi-core processing (PatchMatch)
global algo;                    % PatchMatch algorithm type
global cohFac;                  % weight of patches for the Coherency direction (in BDS). The weight for Completeness is 1-cohFac.
global refInd;                  % a function that returns the index of the reference frame given the number of frame.
global b_max;                   % maximum search window size
global b_min;                   % minimum search window size
global delta_s;                 % the threshold for highly saturated pixels
global delta_n;                 % the threshold for detecting inconsistency in search map calculation
%%% ------------------------------------------------

%%% Parameters initialization ----------------------
patch_w = 7;
minImgSize = 150;
numScales = 6;
DSMethod = 'lanczos3';
vMax = 0.9;
vMin = 0.2;
delta_n = 0.04;
delta_s = 0.99;
b_min = 0.002;
b_max = 0.4;
maxImgSize = 250000;
maxRansacFeatures = 10000;
cohFac = 0.8;
cores = 16;
if cores == 1
    algo = 'cpu';
else
    algo = 'cputiled';
end
maxImVal = 2^16-1;
refInd = @(i, n) mod(i-1, n) + 1;

saveIntermediateResults = false;

if (strcmp(profile, 'normal'))
    maxIter = 12;
    minIter = 3;
    iterStep = 3;
    searchVoteInner = 1;
    decreaseIterFromScale = 1;
elseif (strcmp(profile, 'medium'))
    maxIter = 20;
    minIter = 5;
    iterStep = 3;
    searchVoteInner = 1;
    decreaseIterFromScale = 1;
elseif (strcmp(profile, 'high'))
    maxIter = 20;
    minIter = 5;
    iterStep = 3;
    searchVoteInner = 4;
    decreaseIterFromScale = 4;
end

assert(decreaseIterFromScale > 0 && decreaseIterFromScale <= numScales);

%%% -----------------------------------------