function Initialization(vidObj, resizeFac, expoTimes, fi)

%%% Declaring global variables ------------------
global numScales;
global refInd;
global vMax;
global vMin;
global DSMethod;
global b_max;
%%% ---------------------------------------------


ri = refInd(fi, vidObj.numExposures);
if (ri == 1)
    status = 'high';
elseif (ri == vidObj.numExposures)
    status = 'low';
else
    status = 'both';
end


%%% Computing initial guesses -----------------------------------------
refImgSmall = ReadFromBinaryFile(targetFile(fi, numScales, ri));
for ni = 1 : vidObj.numExposures
    if (ni ~= ri)
        tmp = LDRtoLDR(refImgSmall, expoTimes(ri), expoTimes(ni));
        WriteToBinaryFile(targetFile(fi, numScales, ni), tmp);
    end
end
%%% -------------------------------------------------------------------

%%% Computing all the pyramids for other scales ----------------------- 
refImg = ReadFromBinaryFile(targetFile(fi, 1, ri));

alphaPlus = double(refImg < vMin);
alphaMinus = double(refImg > vMax);

VMax = double(refImg > vMax);
VMin = double(refImg < vMin);

% Computing alpha based using the function shown in Fig. 5
alpha = GetAlpha(refImg, status);
darkRegions = repmat(sum(VMin > 0, 3) == 3, [1,1,3]);

lsMap = ReadFromBinaryFile(lsMapFile(fi, 1));
rsMap = ReadFromBinaryFile(rsMapFile(fi, 1));

winSize = b_max * sqrt(vidObj.h * vidObj.w);
if (isempty(lsMap) || isempty(rsMap))
    highMotion = false(vidObj.h, vidObj.w);
else
    highMotion = lsMap(:, :, 1) >= winSize & rsMap(:, :, 1) >= winSize;
end

highMotion = repmat(highMotion, [1,1,3]);
alpha(highMotion & darkRegions) = alpha(highMotion & darkRegions).^(1/4);
% --------------------------------------------------------


for si = 2 : numScales
    curVMax = max(0,min(1, imresize(VMax, 1/resizeFac^(si-1), DSMethod)));
    curVMin = max(0,min(1, imresize(VMin, 1/resizeFac^(si-1), DSMethod)));
    
    curAlpha = max(0,min(1, imresize(alpha, 1/resizeFac^(si-1), DSMethod)));
    
    curAlphaPlus = max(0,min(1, imresize(alphaPlus, 1/resizeFac^(si-1), DSMethod)));
    curAlphaMinus = max(0,min(1, imresize(alphaMinus, 1/resizeFac^(si-1), DSMethod)));
    
    curVMax = vMax * curVMax;
    curVMin = 1 - (1 - vMin) * curVMin;
    
    WriteToBinaryFile(alphaFile(fi, si), curAlpha);
    WriteToBinaryFile(alphaPlusFile(fi, si), curAlphaPlus);
    WriteToBinaryFile(alphaMinusFile(fi, si), curAlphaMinus);
    WriteToBinaryFile(VMaxFile(fi, si), curVMax);
    WriteToBinaryFile(VMinFile(fi, si), curVMin);
end

VMax = vMax * VMax;
VMin = 1 - (1 - vMin) * VMin;

WriteToBinaryFile(alphaFile(fi, 1), alpha);
WriteToBinaryFile(alphaPlusFile(fi, 1), alphaPlus);
WriteToBinaryFile(alphaMinusFile(fi, 1), alphaMinus);
WriteToBinaryFile(VMaxFile(fi, 1), VMax);
WriteToBinaryFile(VMinFile(fi, 1), VMin);
%%% -------------------------------------------------------------------
