function [vidObj, exposureTimes] = ReadSceneFiles(inputSceneFolder, sceneName, outFinalFolder, profile)

%%% Declaring global parameters ----------------------
global outFinalDir;
global saveIntermediateResults;
global numScales;
global sceneDataPath;
%%% --------------------------------------------------

outFinalDir = [outFinalFolder, '/', sceneName, '_', profile];
inputFolder = sprintf('%s/%s', inputSceneFolder, sceneName);

if(~exist(inputFolder, 'dir'))
    error('The scene directory %s does not exist', inputFolder);
end


%%% Read the exposure information
expoFileName = dir([inputFolder, '/Exposures.txt']);
fileName = sprintf('%s/%s', inputFolder, expoFileName.name);

fid = fopen(fileName);
exposureTimes = 2.^cell2mat(textscan(fid, '%f'));   % convert exposure value to exposure time (2^exposureValue = exposureTime)
fclose(fid);

%%% Creating a video object to read the frames later
vidObj = CreateVideoObject(inputFolder);
vidObj.numExposures = length(exposureTimes);

%%% Creating output directories and intermediate scene data directories
mkdir([outFinalDir '/Matched']);
mkdir([outFinalDir '/HDR']);

sceneDataPath = [inputFolder '/SceneData'];
if (~exist(sceneDataPath, 'dir'))
    mkdir(sceneDataPath);
    for si = 1 : numScales
        for fi = 1 : vidObj.numFrames
            for ni = 1 : vidObj.numExposures
                outEachImgDir = sprintf('%s/%03d/%02d/%d', sceneDataPath, fi, si, ni);
                mkdir(outEachImgDir);
            end
        end
    end
end

%%% Creating intermediate result directories
if (saveIntermediateResults)
    for ni = 1 : vidObj.numExposures
        outEachImgDir = sprintf('%s/Intermediate/%d', outFinalDir, ni);
        mkdir(outEachImgDir);
    end
end



