function ReconstructHDRVideo(vidObj, expoTimes, resizeFac, sceneName)
%%% This code does lines 2 to 15 of Algorithm 1 in the paper

%%% Declaring global variables -------------------
global maxIter;
global numScales;
global outFinalDir;
global saveIntermediateResults;
global vMin;
global vMax;
global b_max;
global maxImVal;
global iterStep;
global minIter;
global decreaseIterFromScale;
global searchVoteInner;
global sceneDataPath;
%%% ----------------------------------------------

%%% Parameters initializations -------------------------
messageLengthScale = 0;
messageLengthInScale = 0;
mLength = 0;
numFrames = vidObj.numFrames;
numExposures = vidObj.numExposures;
h = ceil(vidObj.h / resizeFac^(numScales-1));
w = ceil(vidObj.w / resizeFac^(numScales-1));
maxWin = ceil(b_max * sqrt(h * w));
%%% -----------------------------------------------------


for rsi = 1 : numScales
    
    %%% writing the progress information -------------
    fprintf(repmat('\b', 1, messageLengthScale + messageLengthInScale)); 
    messageLengthScale = fprintf('Scale %d/%d', rsi, numScales);
    %%% ----------------------------------------------
    
    numIterThisScale = maxIter - (rsi - 1) * iterStep;
    numIterThisScale = max(numIterThisScale, minIter);
    messageLengthInScale = 0;
    startTime = clock;
    
    si = numScales - rsi + 1;
    
    if (saveIntermediateResults)
        WriteReferenceImages(vidObj, si);
    end
    
    for iter = 1 : numIterThisScale
        
        %%% writing the progress information ------------
        fprintf(repmat('\b', 1, messageLengthInScale + mLength));
        if (iter ~= 1)
            messageLengthInScale = fprintf('\nIteration %d/%d, finishing in %s', iter, numIterThisScale, Sec2Str((numIterThisScale/iter - 1)*etime(clock, startTime)));
        else
            messageLengthInScale = fprintf('\nIteration %d/%d', iter, numIterThisScale);
        end
        %%% ---------------------------------------------
        
        %%% The whole search and vote process for all the frames.
        rs_max = [];
        
        if (rsi < decreaseIterFromScale)
            numInnerIter = searchVoteInner;
        else
            numInnerIter = 1;
        end
        
        for ii = 1 : numInnerIter
            SearchAndVote(vidObj, rs_max, maxWin, si);
        end
        %%% ---------------------------------------------
        
        %%% Writing the intermediate results
        if (saveIntermediateResults)
            WriteReconstructedTargets(vidObj, si);
        end
        %%% ---------------------------------------------
        
        %%% Performing HDRMerge on all the frames
        
        if (iter ~= numIterThisScale)
            mLength = fprintf('\nPerforming HDR merge ...');
            PerformHDRMergeForAllFrames(vidObj, expoTimes, si);
        end
        %%% ---------------------------------------------

    end
    
    if (rsi ~= numScales)
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% In this part of the code, data is prepared to go one scale up
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        fprintf(repmat('\b', 1, messageLengthScale + messageLengthInScale));
        messageLengthScale = fprintf('Preparing to go one scale up');
        
        h = ceil(vidObj.h / resizeFac ^ (numScales - rsi -1));
        w = ceil(vidObj.w / resizeFac ^ (numScales - rsi -1));
        maxWin = ceil(b_max * sqrt(h * w));
        
        %%% We upsample all the target images except the reference images
        %%% which we directly grab them from the pyramid. The upscale
        %%% targets are then merged and new targets are extracted. We also
        %%% upscale the NNFs.
        UpSampleTargetsAndNNFs(vidObj, expoTimes, si, h, w);            
        
        for fi  = 1 : vidObj.numFrames
            fileName = sprintf('%s/%03d/%02d', sceneDataPath, fi, si);
            rmdir(fileName, 's');
        end

        %%% ---------------------------------------------
        
        %%% Performing search and vote on the new targets with rs_max = 0.
        %%% This means that search is done only with propagation and no
        %%% random search is done. This ensures that we do not deviate a lot
        %%% from the previous calculated NNF.
        rs_max = 0;
        SearchAndVote(vidObj, rs_max, maxWin, si-1);
        %%% ---------------------------------------------
        
        %%% HDR merge of the new targets ----------------
        mLength = fprintf('\nPerforming HDR merge ...');
        PerformHDRMergeForAllFrames(vidObj, expoTimes, si-1);
        %%% ---------------------------------------------
        
        fprintf(repmat('\b', 1, mLength + messageLengthScale));
        messageLengthInScale = 0;
        messageLengthScale = 0;
        mLength = 0;
    
    else
        
        %%% When the code gets to here all the targets are finalized. So we
        %%% run final HDR merge on them which does not perform the
        %%% consistency check.
        ri = ceil((numExposures + 1) / 2);
        
        if (ri == 1)
            status = 'high';
        elseif (ri == numExposures)
            status = 'low';
        else
            status = 'both';
        end
        
        for fi = vidObj.numExposures + 1 : numFrames - vidObj.numExposures
            
            targets = cell(1, numExposures);
            for ni = 1 : numExposures
                targets{ni} = ReadFromBinaryFile(targetFile(fi, 1, ni));
                imwrite(uint16(targets{ni} * maxImVal), [outFinalDir, sprintf('/Matched/%s-l%03d-%d.tif', sceneName, fi, ni)]);
            end
            
            alphaPlus = double(targets{ri} < vMin);
            alphaMinus = double(targets{ri} > vMax);
            alpha = GetAlpha(targets{ri}, status);
            
            [Htilde, ~, ~] = HDRmerge(targets, alphaPlus, alphaMinus, expoTimes, ri);
            Href = LDRtoHDR(targets{ri}, expoTimes(ri));
            H = AlphaBlend(Href, Htilde, alpha);
            
            hdrwrite(H, [outFinalDir, sprintf('/HDR/%s-%03d.hdr', sceneName, fi)]);
            
        end
        
        rmdir(sceneDataPath, 's');
        
    end
end
