function SearchAndVote(vidObj, rs_max, winSize, si)

%%% Declaring global variables
global refInd;
global cohFac;
global patch_w;
%%% -------------------------

comFac = 1 - cohFac;
numExposures = vidObj.numExposures;
numFrames = vidObj.numFrames;
mLength = 0;

%%% -------------------- Search Process --------------------
for fi = 1 : numFrames
    
    %%% writing the progress information -------------
    fprintf(repmat('\b', 1, mLength)); 
    mLength = fprintf('\nPerforming search on frame %d/%d', fi, numFrames);
    %%% ----------------------------------------------
    
    for ni = 1 : numExposures
        
        % Searching using the left image
        if (fi-1 >= 1)
            NNFPrior = ReadFromBinaryFile(lNNFPriorFile(fi, si));
            searchWin = ReadFromBinaryFile(lsMapFile(fi, si));
            NNFPrev = ReadFromBinaryFile(lNNFPrevFile(fi, si, ni));
            source = ReadFromBinaryFile(targetFile(fi-1, si, ni));
            target = ReadFromBinaryFile(targetFile(fi, si, ni));
            
            if (isempty(NNFPrior))
                [NNFcur, cohWeight] = ComputeNNF(target, source, rs_max, winSize, NNFPrev, [], [], fi, si); 
                WriteToBinaryFile(lNNFPrevFile(fi, si, ni), NNFcur);
                WriteToBinaryFile(lCohWeightFile(fi, si, ni), cohWeight);
            else
                [NNFcur, cohWeight] = ComputeNNF(target, source, rs_max, [], NNFPrev, NNFPrior, searchWin, fi, si); 
                WriteToBinaryFile(lNNFPrevFile(fi, si, ni), NNFcur);
                WriteToBinaryFile(lCohWeightFile(fi, si, ni), cohWeight);
            end
        end
        % --------------------------------
        
        % Searching using the right image
        if (fi+1 <= numFrames)
            NNFPrior = ReadFromBinaryFile(rNNFPriorFile(fi, si));
            searchWin = ReadFromBinaryFile(rsMapFile(fi, si));
            NNFPrev = ReadFromBinaryFile(rNNFPrevFile(fi, si, ni));
            source = ReadFromBinaryFile(targetFile(fi+1, si, ni));
            target = ReadFromBinaryFile(targetFile(fi, si, ni));
            
            if (isempty(NNFPrior))
                [NNFcur, cohWeight] = ComputeNNF(target, source, rs_max, winSize, NNFPrev, [], [], fi, si); 
                WriteToBinaryFile(rNNFPrevFile(fi, si, ni), NNFcur);
                WriteToBinaryFile(rCohWeightFile(fi, si, ni), cohWeight);
            else
                [NNFcur, cohWeight] = ComputeNNF(target, source, rs_max, [], NNFPrev, NNFPrior, searchWin, fi, si); 
                WriteToBinaryFile(rNNFPrevFile(fi, si, ni), NNFcur);
                WriteToBinaryFile(rCohWeightFile(fi, si, ni), cohWeight);
            end
        end
        % -----------------------------
        
    end
end
%%% ---------------------------------------------------------

%%% ---------------------- Vote process ---------------------
for fi = 1 : numFrames
    
    %%% writing the progress information -------------
    fprintf(repmat('\b', 1, mLength)); 
    mLength = fprintf('\nPerforming vote on frame %d/%d', fi, numFrames);
    %%% ----------------------------------------------
    
    ri = refInd(fi, numExposures);
    for ni = 1 : numExposures
        if (ni ~= ri)
            
            img = 0;
            
            % Voting the left image
            if (fi-1 >= 1)
                NNFCoh = ReadFromBinaryFile(lNNFPrevFile(fi, si, ni));
                cohWeightCoh = ReadFromBinaryFile(lCohWeightFile(fi, si, ni));
                NNFCom = ReadFromBinaryFile(rNNFPrevFile(fi-1, si, ni));
                cohWeightCom = ReadFromBinaryFile(rCohWeightFile(fi-1, si, ni));
                
                source = ReadFromBinaryFile(targetFile(fi-1, si, ni));
                img = votemex(single(source), int32(NNFCoh), int32(NNFCom), 'cpu', patch_w, [], uint8(cohWeightCom), cohFac, comFac, [], uint8(cohWeightCoh));
            end
            % -----------------------------
            
            % Voting the right image
            if (fi+1 <= numFrames)
                NNFCoh = ReadFromBinaryFile(rNNFPrevFile(fi, si, ni));
                cohWeightCoh = ReadFromBinaryFile(rCohWeightFile(fi, si, ni));
                NNFCom = ReadFromBinaryFile(lNNFPrevFile(fi+1, si, ni));
                cohWeightCom = ReadFromBinaryFile(lCohWeightFile(fi+1, si, ni));
                
                source = ReadFromBinaryFile(targetFile(fi+1, si, ni));
                img = img + votemex(single(source), int32(NNFCoh), int32(NNFCom), 'cpu', patch_w, [], uint8(cohWeightCom), cohFac, comFac, [], uint8(cohWeightCoh));
            end
            % -----------------------------

            avgImg = img(:, :, 1:3)./repmat(img(:, :, 4), [1,1,3]);
            avgImg(repmat(img(:, :, 4), [1,1,3]) == 0) = 0;
            avgImg = Clamp(avgImg ./ 255, 0, 1);

            WriteToBinaryFile(targetTmpFile(fi, si, ni), avgImg);
            
        end
    end
end
%%% ------------------------------------------------------

fprintf(repmat('\b', 1, mLength));

%%% Overwriting the previous target files with the new ones
for fi = 1 : numFrames
    ri = refInd(fi, numExposures);
    for ni = 1 : numExposures
        if (ni ~= ri)
            movefile(targetTmpFile(fi, si, ni),targetFile(fi, si, ni));
        end
    end
end
%%% ----------------------------


