function ToneMap(tonemapperPath, inputSceneFolder, sceneName)

global outFinalDir;

currentPath = pwd;
resultFolder = [currentPath, '/', outFinalDir];



%%% creating output directory and reading the file names
mkdir([resultFolder, '/', 'TMimages']);
fileNames = dir([resultFolder, '/HDR/*.hdr']);
fileNames = {fileNames(:).name};

%%% setting initial parameters
numImages = length(fileNames);
C = makecform('srgb2lab');
epsilon = 1e-6;


%%% Reading the tonemapping parameters from a file
tmInfoFile = [inputSceneFolder, '/', sceneName, '/TMinfo.txt'];
fid = fopen(tmInfoFile, 'rt');
if (fid == -1)
    error('The tonemapping parameter file %s does not exist', tmInfoFile);
end
params = cell2mat(textscan(fid, '%f'));
fclose(fid);

keyFac = params(1);
colorSaturationFac = params(2);
gamma = params(3);


%%% Main loop for tonemapping the images
mLength = 0;
for i = 1 : numImages
    
    %%% first image is treated differently since there is not any frame
    %%% before that
    if (i > 1)
        imgName = sprintf('%s/HDR/%s', resultFolder, fileNames{i});
        img = double(hdrread(imgName));
        imgLab = applycform(img, C);
        logSumCur = sum(log(epsilon + Vec(imgLab(:, :, 1))));
        
        logAverageTemp = exp(1/(2*h*w) * (logSumCur+logSumPrev));
        logAverafeCur = exp(1/(h*w) * logSumCur);
        
        key = keyFac * logAverafeCur/logAverageTemp;
        
        logSumPrev = logSumCur;
        
    else
        %%% the keyfac is computed based on average brightness of this
        %%% frame and previous frame
        imgName = sprintf('%s/HDR/%s', resultFolder, fileNames{i});
        img = double(hdrread(imgName));
        imgLab = applycform(img, C);
        [h, w, ~] = size(img);
        logSumPrev = sum(log(epsilon + Vec(imgLab(:, :, 1))));
        key = keyFac;
    end
    
    %%% running reihnard tonemapper
    command = sprintf('luminance-hdr-cli.exe -l %s/HDR/%s -t reinhard02 -p key=%f -g %f -o %s/TMimages/%s-%03d.tif', resultFolder, fileNames{i}, key, gamma, resultFolder, sceneName, i);
    cd(tonemapperPath);
    system(command);
    cd(currentPath);
    
    %%% color saturation adjustment
    imgPath = sprintf('%s/TMimages/%s-%03d.tif', resultFolder, sceneName, i);
    img = imread(imgPath);
    hsvImg = rgb2hsv(img);
    hsvImg(:,:,2) = hsvImg(:,:,2) * colorSaturationFac;
    img = hsv2rgb(hsvImg);
    delete(imgPath);
    imgPath = sprintf('%s/TMimages/Img-%03d.jpg', resultFolder, i);
    imwrite(img, imgPath, 'Quality', 100);
    
    fprintf(repmat('\b', [1, mLength + 2]));
    mLength = fprintf('\nTonemapping %2.1f%%', i/numImages * 100);
end

