function validatedFlow = ValidateFlow(curFlow, flow1, flow2)

%%% curFlow is the flow to be checked by concatenating flow1 and flow2 and
%%% check for the consistency.

[h, w, ~] = size(curFlow);

%%% The window size should be multiplied by the size of the image
global b_min;
validWinSize = b_min * sqrt(h * w);

curFlowX = curFlow(:, :, 1);
curFlowY = curFlow(:, :, 2);
flow1X = flow1(:, :, 1);
flow1Y = flow1(:, :, 2);
flow2X = flow2(:, :, 1);
flow2Y = flow2(:, :, 2);

%%% Computing the absolute flow to find the invalid flows
[X, Y] = meshgrid(1:w, 1:h);
flow1X_abs = round(X + flow1X);
flow1Y_abs = round(Y + flow1Y);

invalidFlows = flow1X_abs < 1 | flow1X_abs > w | flow1Y_abs < 1 | flow1Y_abs > h;
flow1X_abs(invalidFlows) = [];
flow1Y_abs(invalidFlows) = [];

%%% Find the indices of the flows at the begining of the flow and at the
%%% halfway point.
halfInd = sub2ind([h, w], flow1Y_abs(:), flow1X_abs(:));
baseInd = sub2ind([h, w], Y(~invalidFlows), X(~invalidFlows));

%%% Concatenating the flows --------------------------------
concatX = inf * ones(h, w);
concatY = inf * ones(h, w);

concatX(baseInd) = flow1X(baseInd) + flow2X(halfInd);
concatY(baseInd) = flow1Y(baseInd) + flow2Y(halfInd);
%%% --------------------------------------------------------

%%% Finding invalid flows
invalidFlows =  max( abs(curFlowX - concatX), abs(curFlowY - concatY) ) > validWinSize;

validatedFlow = curFlow;
validatedFlow(invalidFlows) = 0;
validatedFlow(find(invalidFlows)+ h * w) = 0;